import logging
from langchain_openai import ChatOpenAI
from langchain_core.messages.ai import AIMessage
from .helpers import DotSpinner
from .settings import config


logger = logging.getLogger(__name__)
httpx_logger = logging.getLogger("httpx")
httpx_logger.setLevel(logging.WARNING)

def chat_completion_request(messages, functions=None) -> AIMessage:
    """Sends a request to OpenAI."""

    spinner = DotSpinner()
    spinner.start()
    response = None
    try:
        model = ChatOpenAI(model=config.gpt_model, temperature=0, base_url=config.model_base_url)
        response = model.invoke(messages) if not functions else model.bind_tools(functions).invoke(messages)

    except Exception as e:
        logger.critical(f"Error generating response from API: {e}")
        logger.debug("Stack trace:", exc_info=True)
        exit(1)

    finally:
        spinner.stop()

    logger.debug("Chat completion request successfully executed.")
    return response
