import logging
import os
from typing import Any
from ..helpers import read_file_content

logger = logging.getLogger(__name__)

class FileOperations:
    def __init__(self, project_abspath: str):
        self.project_abspath = project_abspath

    def manage_files_on_disk(self, arguments: Any):
        result = []
        operations = {
            'create': ('created', 'w'),
            'read': ('read', None),
            'update': ('updated', 'w'),
            'delete': ('deleted', None)
        }

        for file in arguments['files']:
            filename = file['filename']
            file_path = os.path.join(self.project_abspath, filename)
            content = file.get('content', '')
            operation = file['operation']

            if operation in operations:
                action, mode = operations[operation]

                if operation == 'create':
                    self._create_file(file_path, content)
                    action = 'directory created' if content == '' else action

                if operation == 'delete':
                    action = self._delete_file(file_path)

                if mode and content != '':
                    self._write_to_file(file_path, content, mode)

                if operation == 'read':
                    content = self._read_file(file_path)
                    result.append(f"{filename}: {content}")
                else:
                    result.append(f"{filename}: {action}")

                logger.info(f"{filename}: {action}")
        return "\n".join(result)

    def _create_file(self, file_path, content):
        if content == '':
            os.makedirs(file_path, exist_ok=True)
        else:
            os.makedirs(os.path.dirname(file_path), exist_ok=True)

    def _delete_file(self, file_path):
        if os.path.exists(file_path):
            if os.path.isdir(file_path):
                os.rmdir(file_path)
            else:
                os.remove(file_path)
            return 'deleted'
        return 'does not exist'

    def _write_to_file(self, file_path, content, mode):
        if not os.path.isdir(file_path):
            with open(file_path, mode, encoding='utf-8') as f:
                f.write(content)
        else:
            logger.error(f"{file_path} is a directory, cannot open it as a file.")

    def _read_file(self, file_path):
        if not os.path.isdir(file_path):
            return read_file_content(file_path)
        logger.error(f"{file_path} is a directory, cannot read it as a file.")
        return ""
